@@ -15,13 +15,13 @@ DIRNAME = os.path.dirname(__file__)
 ICON_APP = os.path.join(DIRNAME, 'resources/iconmonstr-location-27-32.png')
 ICON_CONN = os.path.join(DIRNAME, 'resources/iconmonstr-link-8-24.png')
 ICON_DISCONN = os.path.join(DIRNAME, 'resources/iconmonstr-link-10-24.png')
-ICON_GO = os.path.join(DIRNAME, 'resources/iconmonstr-media-control-4-32.png')
-ICON_STOP = os.path.join(DIRNAME, 'resources/iconmonstr-media-control-12-32.png')
 ICON_POS = os.path.join(DIRNAME, 'resources/iconmonstr-location-1-24.png')
 ICON_SEND = os.path.join(DIRNAME, 'resources/iconmonstr-arrow-12-24.png')
 ICON_EXIT = os.path.join(DIRNAME, 'resources/iconmonstr-door-6-24.png')
 ICON_PENDING = os.path.join(DIRNAME, 'resources/iconmonstr-time-6-24.png')
 ICON_CONFIRMED = os.path.join(DIRNAME, 'resources/iconmonstr-check-mark-8-24.png')
+ICON_WARNING = os.path.join(DIRNAME, 'resources/iconmonstr-warning-1-24.png')
+ICON_UBXCONFIG = os.path.join(DIRNAME, 'resources/iconmonstr-gear-2-32.png')
 IMG_WORLD = os.path.join(DIRNAME, 'resources/world.png')
 BTN_CONNECT = "\u25b6"  # text on "Connected" button
 BTN_DISCONNECT = "\u2587"  # text on "Disconnected" button
