@@ -7,22 +7,24 @@ Created on 22 Sep 2020
 
 @author: semuadmin
 '''
+# pylint: disable=invalid-name, unused-argument
 
-from tkinter import ttk, Toplevel, Frame, Checkbutton, Radiobutton, Listbox, \
-                    Spinbox, Scrollbar, messagebox, Button, Label, StringVar, \
+from tkinter import ttk, messagebox, Toplevel, Frame, Checkbutton, Radiobutton, Listbox, \
+                    Spinbox, Scrollbar, Button, Label, StringVar, \
                     IntVar, N, S, E, W, LEFT, VERTICAL
 
 from PIL import ImageTk, Image
 from pyubx2 import UBXMessage, POLL, SET, UBX_CONFIG_MESSAGES
 
-from .globals import BGCOL, FGCOL, ENTCOL, ICON_APP, ICON_SEND, ICON_EXIT, \
+from .globals import BGCOL, FGCOL, ENTCOL, ICON_APP, ICON_SEND, ICON_EXIT, ICON_WARNING, \
                      ICON_PENDING, ICON_CONFIRMED, BAUDRATES, READONLY
-from .strings import DLGUBXCONFIG
+from .strings import LBLUBXCONFIG, LBLCFGPRT, LBLCFGMSG, LBLPRESET, DLGUBXCONFIG, DLGRESET, \
+                     DLGSAVE, DLGRESETCONFIRM, DLGSAVECONFIRM
 from .ubx_handler import UBXHandler as ubh, CFG_MSG_OFF, CFG_MSG_ON
 
 MSG_PRESETS = {
 'CFG-CFG - RESTORE FACTORY DEFAULTS': 'CFG-CFG',
-'CFG-CFG - Store current configuration': 'CFG-CFG',
+'CFG-CFG - Save current configuration to RAM': 'CFG-CFG',
 'CFG-MSG - Turn ON minimum NMEA msgs': 'CFG-MSG',
 'CFG-MSG - Turn ON all NMEA msgs': 'CFG-MSG',
 'CFG-MSG - Turn OFF all NMEA msgs': 'CFG-MSG',
@@ -62,79 +64,27 @@ class UBXConfigDialog():
         wa = self.__master.winfo_width()
         ha = self.__master.winfo_height()
         self._dialog.geometry("+%d+%d" % (int(wa / 2 - wd / 2), int(ha / 2 - hd / 2)))
-#         self._dialog.attributes('true')
         self._img_icon = ImageTk.PhotoImage(Image.open (ICON_APP))
         self._img_pending = ImageTk.PhotoImage(Image.open(ICON_PENDING))
         self._img_confirmed = ImageTk.PhotoImage(Image.open(ICON_CONFIRMED))
         self._img_send = ImageTk.PhotoImage(Image.open(ICON_SEND))
         self._img_exit = ImageTk.PhotoImage(Image.open(ICON_EXIT))
-        self._awaiting_ack = False
-
-        # Poll current UBX configuration
-        if self.__app.serial_handler.connected:
-            ubh.poll_ubx_config(self.__app.serial_handler.serial)
-
-        # Load user presets if there are any
-        self._userpresets = self.__app.file_handler.load_user_presets()
+        self._img_warn = ImageTk.PhotoImage(Image.open(ICON_WARNING))
 
         #  Initialise up key variables
+        self._awaiting_cfgmsg = False
+        self._awaiting_ack = False
+        self._cfg_msg_command = None
         self._preset_command = None
-        self._status = StringVar()
+        self._status_preset = StringVar()
+        self._status_cfgmsg = StringVar()
         self._ubx_baudrate = StringVar()
         self._ubx_inprot = IntVar()
         self._ubx_outprot = IntVar()
-        self._dtm_state = IntVar()
-        self._gbs_state = IntVar()
-        self._gga_state = IntVar()
-        self._gll_state = IntVar()
-        self._gsa_state = IntVar()
-        self._gsv_state = IntVar()
-        self._rmc_state = IntVar()
-        self._txt_state = IntVar()
-        self._vtg_state = IntVar()
-        self._zda_state = IntVar()
-        self._xxx_state = IntVar()
-        self._ubx00_state = IntVar()
-        self._ubx03_state = IntVar()
-        self._ubx04_state = IntVar()
-        self._ubx05_state = IntVar()
-        self._ubx06_state = IntVar()
-        self._navaopstatus_state = IntVar()
-        self._navdop_state = IntVar()
-        self._navposllh_state = IntVar()
-        self._navpvt_state = IntVar()
-        self._navsol_state = IntVar()
-        self._navsvinfo_state = IntVar()
-        self._navsbas_state = IntVar()
-        self._navtimeutc_state = IntVar()
-        self._navvelned_state = IntVar()
-
-#         self._cfg_msg_states = {
-#         'DTM': self._dtm_state,
-#         'GBS': self._gbs_state,
-#         'GGA': self._gga_state,
-#         'GLL': self._gll_state,
-#         'GSA': self._gsa_state,
-#         'GSV': self._gsv_state,
-#         'RMC': self._rmc_state,
-#         'TXT': self._txt_state,
-#         'VTG': self._vtg_state,
-#         'ZDA': self._zda_state,
-#         'XXX': self._xxx_state,  # for error testing
-#         'UBX00': self._ubx00_state,
-#         'UBX03': self._ubx03_state,
-#         'UBX04': self._ubx04_state,
-#         'UBX05': self._ubx05_state,
-#         'UBX06': self._ubx06_state,
-#         'NAV-AOPSTATUS': self._navaopstatus_state,
-#         'NAV-DOP': self._navdop_state,
-#         'NAV-POSLLH': self._navposllh_state,
-#         'NAV-PVT': self._navpvt_state,
-#         'NAV-SOL': self._navsol_state,
-#         'NAV-SVINFO': self._navsvinfo_state,
-#         'NAV-SBAS': self._navsbas_state,
-#         'NAV-TIMEUTC': self._navtimeutc_state,
-#         'NAV-VELNED': self._navvelned_state}
+        self._ddc_onoff = IntVar()
+        self._uart1_onoff = IntVar()
+        self._usb_onoff = IntVar()
+        self._spi_onoff = IntVar()
 
         self._body()
         self._do_layout()
@@ -155,9 +105,12 @@ class UBXConfigDialog():
 
         con.option_add("*Font", self.__app.font_sm)
 
-        self._lbl_title = Label(con, text="UBX Configuration", bg=BGCOL, fg=FGCOL,
-                                justify=LEFT, font=self.__app.font_md)
-        self._lbl_cfg_port = Label(con, text="CFG-PRT Port and Protocol Configuration",
+        self._lbl_title = Label(con, text=LBLUBXCONFIG, bg=BGCOL, fg=FGCOL,
+                                justify=LEFT, font=self.__app.font_md, width=50)
+        # *******************************************************
+        # Port Configuration
+        # *******************************************************
+        self._lbl_cfg_port = Label(con, text=LBLCFGPRT,
                                    bg=LBL_COL, anchor='w')
         self._lbl_ubx_baudrate = Label(con, text="Baud rate")
         self._spn_ubx_baudrate = Spinbox(con,
@@ -185,102 +138,32 @@ class UBXConfigDialog():
                                      command=self._on_send_port,
                                      font=self.__app.font_md)
 
-        self._lbl_cfg_msg = Label(con, text="CFG-MSG - NMEA Message Filters",
+        # *******************************************************
+        # CFG-MSG Message Selection
+        # *******************************************************
+        self._lbl_cfg_msg = Label(con, text=LBLCFGMSG,
                                   bg=LBL_COL, anchor='w')
-        self._chk_dtm = Checkbutton(con, text="DTM",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x0A',
-                                                                     var=self._dtm_state),
-                                    variable=self._dtm_state)
-        self._chk_gbs = Checkbutton(con, text="GBS",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x09',
-                                                                     var=self._gbs_state),
-                                    variable=self._gbs_state)
-        self._chk_gll = Checkbutton(con, text="GLL",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x01',
-                                                                     var=self._gll_state),
-                                    variable=self._gll_state)
-        self._chk_gga = Checkbutton(con, text="GGA",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x00',
-                                                                     var=self._gga_state),
-                                    variable=self._gga_state)
-        self._chk_gsa = Checkbutton(con, text="GSA",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x02',
-                                                                     var=self._gsa_state),
-                                    variable=self._gsa_state)
-        self._chk_gsv = Checkbutton(con, text="GSV",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x03',
-                                                                     var=self._gsv_state),
-                                    variable=self._gsv_state)
-        self._chk_rmc = Checkbutton(con, text="RMC",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x04',
-                                                                     var=self._rmc_state),
-                                    variable=self._rmc_state)
-        self._chk_vtg = Checkbutton(con, text="VTG",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x05',
-                                                                     var=self._vtg_state),
-                                    variable=self._vtg_state)
-        self._chk_txt = Checkbutton(con, text="TXT",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x41',
-                                                                     var=self._txt_state),
-                                    variable=self._txt_state)
-        self._chk_zda = Checkbutton(con, text="ZDA",
-                                    command=lambda: self._on_cfg_msg(msg=b'\xF0\x08',
-                                                                     var=self._zda_state),
-                                    variable=self._zda_state)
-        self._chk_ubx00 = Checkbutton(con, text="UBX,00",
-                                      command=lambda: self._on_cfg_msg(msg=b'\xF1\x00',
-                                                                       var=self._ubx00_state),
-                                      variable=self._ubx00_state)
-        self._chk_ubx03 = Checkbutton(con, text="UBX,03",
-                                      command=lambda: self._on_cfg_msg(msg=b'\xF1\x03',
-                                                                       var=self._ubx03_state),
-                                      variable=self._ubx03_state)
-        self._chk_ubx04 = Checkbutton(con, text="UBX,04",
-                                      command=lambda: self._on_cfg_msg(msg=b'\xF1\x04',
-                                                                       var=self._ubx04_state),
-                                      variable=self._ubx04_state)
-        self._lbl_ubx_msg = Label(con, text="CFG-MSG - UBX NAV Message Filters",
-                                  bg=LBL_COL, anchor='w')
-        self._chk_navaopstatus = Checkbutton(con, text="AOPSTATUS",
-                                    command=lambda: self._on_cfg_msg(msg=b'\x01\x03',
-                                                                     var=self._navaopstatus_state),
-                                    variable=self._navaopstatus_state)
-        self._chk_navsol = Checkbutton(con, text="SOL",
-                                    command=lambda: self._on_cfg_msg(msg=b'\x01\x06',
-                                                                     var=self._navsol_state),
-                                    variable=self._navsol_state)
-        self._chk_navdop = Checkbutton(con, text="DOP",
-                                    command=lambda: self._on_cfg_msg(msg=b'\x01\x04',
-                                                                     var=self._navdop_state),
-                                    variable=self._navdop_state)
-        self._chk_navposllh = Checkbutton(con, text="POSLLH",
-                                    command=lambda: self._on_cfg_msg(msg=b'\x01\x02',
-                                                                     var=self._navposllh_state),
-                                    variable=self._navposllh_state)
-        self._chk_navpvt = Checkbutton(con, text="PVT",
-                                    command=lambda: self._on_cfg_msg(msg=b'\x01\x07',
-                                                                     var=self._navpvt_state),
-                                    variable=self._navpvt_state)
-        self._chk_navsvinfo = Checkbutton(con, text="SVINFO",
-                                    command=lambda: self._on_cfg_msg(msg=b'\x01\x30',
-                                                                     var=self._navsvinfo_state),
-                                    variable=self._navsvinfo_state)
-        self._chk_navsbas = Checkbutton(con, text="SBAS",
-                                      command=lambda: self._on_cfg_msg(msg=b'\x01\x32',
-                                                                       var=self._navsbas_state),
-                                      variable=self._navsbas_state)
-        self._chk_navvelned = Checkbutton(con, text="VELNED",
-                                      command=lambda: self._on_cfg_msg(msg=b'\x01\x12',
-                                                                       var=self._navvelned_state),
-                                      variable=self._navvelned_state)
-        self._chk_navtimeutc = Checkbutton(con, text="TIMEUTC",
-                                      command=lambda: self._on_cfg_msg(msg=b'\x01\x21',
-                                                                       var=self._navtimeutc_state),
-                                      variable=self._navtimeutc_state)
+        self._lbx_cfg_msg = Listbox(con, border=2,
+                                 relief="sunken", bg=ENTCOL,
+                                 height=7, justify=LEFT,
+                                 exportselection=False)
+        self._scr_cfg_msg = Scrollbar(con, orient=VERTICAL)
+        self._lbx_cfg_msg.config(yscrollcommand=self._scr_cfg_msg.set)
+        self._scr_cfg_msg.config(command=self._lbx_cfg_msg.yview)
+        self._chk_ddc = Checkbutton(con, text='DDC (I2C)', variable=self._ddc_onoff)
+        self._chk_uart1 = Checkbutton(con, text='UART1', variable=self._uart1_onoff)
+        self._chk_usb = Checkbutton(con, text='USB', variable=self._usb_onoff)
+        self._chk_spi = Checkbutton(con, text='SPI', variable=self._spi_onoff)
+        self._lbl_send_cfg_msg = Label(con)
+        self._btn_send_cfg_msg = Button(con, image=self._img_send, width=50, fg="green",
+                                       command=self._on_send_cfg_msg, font=self.__app.font_md)
+        self._lbl_status_cfg_msg = Label(con, textvariable=self._status_cfgmsg, anchor='w')
 
-        self._lbl_presets = Label(con, text="Preset UBX Configuration Commands",
+        # *******************************************************
+        # PRESET Message Selection
+        # *******************************************************
+        self._lbl_presets = Label(con, text=LBLPRESET,
                                   bg=LBL_COL, anchor='w')
-        self._lbl_preset = Label(con, text="Preset")
         self._lbx_preset = Listbox(con, border=2,
                                  relief="sunken", bg=ENTCOL,
                                  height=7, justify=LEFT,
@@ -288,11 +171,11 @@ class UBXConfigDialog():
         self._scr_preset = Scrollbar(con, orient=VERTICAL)
         self._lbx_preset.config(yscrollcommand=self._scr_preset.set)
         self._scr_preset.config(command=self._lbx_preset.yview)
-
         self._lbl_send_preset = Label(con)
         self._btn_send_preset = Button(con, image=self._img_send, width=50, fg="green",
                                        command=self._on_send_preset, font=self.__app.font_md)
-        self._lbl_status = Label(con, textvariable=self._status, anchor='w')
+
+        self._lbl_status = Label(con, textvariable=self._status_preset, anchor='w')
         self._btn_exit = Button(con, image=self._img_exit, width=50, fg="red",
                                 command=self._on_exit, font=self.__app.font_md)
 
@@ -321,36 +204,27 @@ class UBXConfigDialog():
         self._lbl_send_port.grid(column=5, row=2, ipadx=3,
                                  ipady=3, sticky=(W))
 
+        # *******************************************************
+        # CFG-MSG Message Selection
+        # *******************************************************
         ttk.Separator(self._frm_container).grid(column=0, row=5, columnspan=6,
                                                 padx=3, pady=3, sticky=(W, E))
         self._lbl_cfg_msg.grid(column=0, row=6, columnspan=6, padx=3, sticky=(W, E))
-        self._chk_dtm.grid(column=0, row=7, sticky=(W))
-        self._chk_gbs.grid(column=1, row=7, sticky=(W))
-        self._chk_gga.grid(column=2, row=7, sticky=(W))
-        self._chk_gll.grid(column=3, row=7, sticky=(W))
-        self._chk_gsa.grid(column=4, row=7, sticky=(W))
-        self._chk_gsv.grid(column=5, row=7, sticky=(W))
-        self._chk_rmc.grid(column=0, row=8, sticky=(W))
-        self._chk_txt.grid(column=1, row=8, sticky=(W))
-        self._chk_vtg.grid(column=2, row=8, sticky=(W))
-        self._chk_zda.grid(column=3, row=8, sticky=(W))
-        self._chk_ubx00.grid(column=0, row=9, sticky=(W))
-        self._chk_ubx03.grid(column=1, row=9, sticky=(W))
-        self._chk_ubx04.grid(column=2, row=9, sticky=(W))
-
-        ttk.Separator(self._frm_container).grid(column=0, row=10, columnspan=6,
-                                                padx=3, pady=3, sticky=(W, E))
-        self._lbl_ubx_msg.grid(column=0, row=11, columnspan=6, padx=3, sticky=(W, E))
-        self._chk_navaopstatus.grid(column=0, row=12, sticky=(W))
-        self._chk_navdop.grid(column=1, row=12, sticky=(W))
-        self._chk_navposllh.grid(column=2, row=12, sticky=(W))
-        self._chk_navpvt.grid(column=3, row=12, sticky=(W))
-        self._chk_navsbas.grid(column=4, row=12, sticky=(W))
-        self._chk_navsol.grid(column=5, row=12, sticky=(W))
-        self._chk_navsvinfo.grid(column=0, row=13, sticky=(W))
-        self._chk_navtimeutc.grid(column=1, row=13, sticky=(W))
-        self._chk_navvelned.grid(column=2, row=13, sticky=(W))
+        self._lbx_cfg_msg.grid(column=0, row=7, columnspan=3, rowspan=4, padx=3,
+                               pady=3, sticky=(W, E))
+        self._scr_cfg_msg.grid(column=2, row=7, rowspan=4, sticky=(N, S, E))
+        self._chk_ddc.grid(column=3, row=7, padx=0, pady=0, sticky=(W))
+        self._chk_uart1.grid(column=3, row=8, padx=0, pady=0, sticky=(W))
+        self._chk_usb.grid(column=3, row=9, padx=0, pady=0, sticky=(W))
+        self._chk_spi.grid(column=3, row=10, padx=0, pady=0, sticky=(W))
+        self._btn_send_cfg_msg.grid(column=4, row=7, rowspan=4, ipadx=3, ipady=3,
+                                   sticky=(E))
+        self._lbl_send_cfg_msg.grid(column=5, row=7, rowspan=4, ipadx=3,
+                                 ipady=3, sticky=(W))
 
+        # *******************************************************
+        # PRESET Message Selection
+        # *******************************************************
         ttk.Separator(self._frm_container).grid(column=0, row=14, columnspan=6,
                                                 padx=3, pady=3, sticky=(W, E))
         self._lbl_presets.grid(column=0, row=15, columnspan=6, padx=3, sticky=(W, E))
@@ -371,16 +245,53 @@ class UBXConfigDialog():
 
     def _attach_events(self):
         '''
-        Bind events
+        Bind listbox selection events.
         '''
 
+        self._lbx_cfg_msg.bind("<<ListboxSelect>>", self._on_select_cfg_msg)
         self._lbx_preset.bind("<<ListboxSelect>>", self._on_select_preset)
 
+    def _reset(self):
+        '''
+        Reset settings to defaults.
+        '''
+
+        # Poll current port & usb configuration
+        if self.__app.serial_handler.connected:
+            ubh.poll_ubx_config(self.__app.serial_handler.serial)
+
+        # Load user-defined presets if there are any
+        self._userpresets = self.__app.file_handler.load_user_presets()
+
+        self._ubx_baudrate.set(str(self.__app.ubx_handler.ubx_baudrate))
+        self._ubx_inprot.set(self.__app.ubx_handler.ubx_inprot)
+        self._ubx_outprot.set(self.__app.ubx_handler.ubx_outprot)
+
+        idx = 1
+        for _, val in UBX_CONFIG_MESSAGES.items():
+            self._lbx_cfg_msg.insert(idx, val)
+            idx += 1
+
+        idx = 1
+        for pre in MSG_PRESETS:
+            self._lbx_preset.insert(idx, pre)
+            idx += 1
+
+        for upt in self._userpresets:
+            self._lbx_preset.insert(idx, "USER " + upt)
+            idx += 1
+
     def update(self, cfgtype='CFG-MSG', **kwargs):
         '''
-        Update panel with latest polled configuration
+        Receives polled confirmation messages from the uxb_handler and
+        updates panel settings according to the information received.
+
+        NB: not absolutely bullet-proof as confirmation messages cannot
+        be unequivocally linked to specific commands, but the best
+        available indication of current state.
         '''
 
+        # CFG-PRT command confirmation
         if 'baudrate' in kwargs:
             self._ubx_baudrate.set(str(kwargs['baudrate']))
         if 'inprot' in kwargs:
@@ -388,62 +299,35 @@ class UBXConfigDialog():
         if 'outprot' in kwargs:
             self._ubx_outprot.set(kwargs['outprot'])
         self._lbl_send_port.config(image=self._img_confirmed)
-#         self._lbl_send_preset.config(image=self._img_confirmed)
-        if self._awaiting_ack and cfgtype == 'ACK-ACK':
-            self.set_status(f"{cfgtype} acknowledgement received", "green")
-            self._awaiting_ack = False
-        if self._awaiting_ack and cfgtype == 'ACK-NAK':
-            self.set_status(f"{cfgtype} rejection received", "red")
-            self._awaiting_ack = False
+
+        # CFG-MSG command confirmation
+        if self._awaiting_cfgmsg:
+            if cfgtype == 'CFG-MSG':
+                self.set_status(f"{cfgtype} received", "green")
+                self._ddc_onoff.set(1 if kwargs['ddcrate'] > 0 else 0)
+                self._uart1_onoff.set(1 if kwargs['uart1rate'] > 0 else 0)
+                self._usb_onoff.set(1 if kwargs['usbrate'] > 0 else 0)
+                self._spi_onoff.set(1 if kwargs['spirate'] > 0 else 0)
+                self._lbl_send_cfg_msg.config(image=self._img_confirmed)
+                self._awaiting_cfgmsg = False
+            if cfgtype == 'ACK-NAK':
+                self.set_status(f"{cfgtype} rejected", "red")
+                self._lbl_send_cfg_msg.config(image=self._img_warn)
+                self._awaiting_cfgmsg = False
+
+        # PRESET command(s) confirmation
+        if self._awaiting_ack:
+            if cfgtype == 'ACK-ACK':
+                self.set_status(f"{cfgtype} acknowledgement received", "green")
+                self._lbl_send_preset.config(image=self._img_confirmed)
+                self._awaiting_ack = False
+            if cfgtype == 'ACK-NAK':
+                self.set_status(f"{cfgtype} acknowledgement received", "red")
+                self._lbl_send_preset.config(image=self._img_confirmed)
+                self._awaiting_ack = False
         elif not self._awaiting_ack and cfgtype not in ('ACK-ACK', 'ACK-NAK'):
             self.set_status(f"{cfgtype} response received", "green")
 
-    def _reset(self):
-        '''
-        Reset settings to defaults.
-        '''
-
-        self._ubx_baudrate.set(str(self.__app.ubx_handler.ubx_baudrate))
-        self._ubx_inprot.set(self.__app.ubx_handler.ubx_inprot)
-        self._ubx_outprot.set(self.__app.ubx_handler.ubx_outprot)
-        self._gbs_state.set(False)
-        self._dtm_state.set(False)
-        self._gbs_state.set(False)
-        self._gll_state.set(True)
-        self._gga_state.set(True)
-        self._gsa_state.set(True)
-        self._gsv_state.set(True)
-        self._rmc_state.set(True)
-        self._vtg_state.set(True)
-        self._txt_state.set(True)
-        self._zda_state.set(False)
-        self._ubx00_state.set(False)
-        self._ubx03_state.set(False)
-        self._ubx04_state.set(False)
-
-        idx = 0
-        for pre in MSG_PRESETS:
-            idx += 1
-            self._lbx_preset.insert(idx, pre)
-        for userpre in self._userpresets:
-            idx += 1
-            self._lbx_preset.insert(idx, userpre)
-
-    def _on_cfg_msg(self, *args, **kwargs):
-        '''
-        Send CFG_MSG message to turn individual NMEA message types on or off
-        '''
-
-        msg = kwargs['msg']
-        state = kwargs['var'].get()
-        if state:  # if checkbutton checked
-            onoff = CFG_MSG_ON
-        else:
-            onoff = CFG_MSG_OFF
-        msg_payload = msg + onoff
-        data = UBXMessage('CFG', 'CFG-MSG', msg_payload, 1).serialize()
-        self.__app.serial_handler.serial_write(data)
-
     def _on_send_port(self, *args, **kwargs):
         '''
         Handle Send port config button press.
@@ -467,6 +351,43 @@ class UBXConfigDialog():
         self._lbl_send_port.config(image=self._img_pending)
         self.set_status("CFG-PRT configuration message(s) sent")
 
+    def _on_select_cfg_msg(self, *args, **kwargs):
+        '''
+        CFG-MSG command has been selected.
+        '''
+
+        idx = self._lbx_cfg_msg.curselection()
+        self._cfg_msg_command = self._lbx_cfg_msg.get(idx)
+
+        # poll selected message configuration to get current message rates
+        msg = UBXMessage.key_from_val(UBX_CONFIG_MESSAGES, self._cfg_msg_command)
+        data = UBXMessage('CFG', 'CFG-MSG', msg, POLL)
+        self.__app.serial_handler.serial_write(data.serialize())
+        self._awaiting_cfgmsg = True
+        self._lbl_send_cfg_msg.config(image=self._img_pending)
+
+    def _on_send_cfg_msg(self):
+        '''
+        CFG-MSG command send button has been clicked.
+        '''
+
+        msg = UBXMessage.key_from_val(UBX_CONFIG_MESSAGES, self._cfg_msg_command)
+        rateDDC = b'\x00' if self._ddc_onoff.get() == 0 else b'\x01'
+        rateUART1 = b'\x00' if self._uart1_onoff.get() == 0 else b'\x01'
+        rateUART2 = b'\x00'
+        rateUSB = b'\x00' if self._usb_onoff.get() == 0 else b'\x01'
+        rateSPI = b'\x00' if self._spi_onoff.get() == 0 else b'\x01'
+        reserved = b'\x00'
+        msg_payload = msg + rateDDC + rateUART1 + rateUART2 + rateUSB + rateSPI + reserved
+        data = UBXMessage('CFG', 'CFG-MSG', msg_payload, SET)
+        self.__app.serial_handler.serial_write(data.serialize())
+        data = UBXMessage('CFG', 'CFG-MSG', msg, POLL)  # poll for a response
+        self.__app.serial_handler.serial_write(data.serialize())
+
+        self.set_status("CFG-MSG command sent")
+        self._awaiting_cfgmsg = True
+        self._lbl_send_cfg_msg.config(image=self._img_pending)
+
     def _on_select_preset(self, *args, **kwargs):
         '''
         Preset command has been selected.
@@ -477,49 +398,61 @@ class UBXConfigDialog():
 
     def _on_send_preset(self, *args, **kwargs):
         '''
-        Handle Send preset button press.
+        Preset command send button has been clicked.
         '''
 
-        if self._preset_command == 'CFG-CFG - RESTORE FACTORY DEFAULTS':
-            self._do_factory_reset()
-        elif self._preset_command == 'CFG-INF - Turn ON all INF msgs':
-            self._do_set_inf(True)
-        elif self._preset_command == 'CFG-INF - Turn OFF all non-error INF msgs':
-            self._do_set_inf(False)
-        elif self._preset_command == 'CFG-MSG - Turn ON all LOG msgs':
-            self._do_set_log(True)
-        elif self._preset_command == 'CFG-MSG - Turn OFF all LOG msgs':
-            self._do_set_log(False)
-        elif self._preset_command == 'CFG-MSG - Turn ON all MON msgs':
-            self._do_set_mon(True)
-        elif self._preset_command == 'CFG-MSG - Turn OFF all MON msgs':
-            self._do_set_mon(False)
-        elif self._preset_command == 'CFG-MSG - Turn ON all RXM msgs':
-            self._do_set_rxm(True)
-        elif self._preset_command == 'CFG-MSG - Turn OFF all RXM msgs':
-            self._do_set_rxm(False)
-        elif self._preset_command == 'CFG-MSG - Turn ON minimum NMEA msgs':
-            self._do_set_minnmea()
-        elif self._preset_command == 'CFG-MSG - Turn ON all NMEA msgs':
-            self._do_set_allnmea(True)
-        elif self._preset_command == 'CFG-MSG - Turn OFF all NMEA msgs':
-            self._do_set_allnmea(False)
-        elif self._preset_command == 'CFG-MSG - Turn ON minimum UBX NAV msgs':
-            self._do_set_minNAV()
-        elif self._preset_command == 'CFG-MSG - Turn ON all UBX NAV msgs':
-            self._do_set_allNAV(True)
-        elif self._preset_command == 'CFG-MSG - Turn OFF all UBX NAV msgs':
-            self._do_set_allNAV(False)
-        elif self._preset_command == 'CFG-PRT - Poll Port config':
-            self._do_poll_prt()
-        elif self._preset_command == 'CFG-INF - Poll Info message config':
-            self._do_poll_inf()
-        else:
-            messagebox.showwarning("UBX Configuration Presets", "Sorry - not yet implemented!\n\nWatch this space.",)
+        confirmed = True
+        try:
 
-        self.set_status("Configuration message(s) sent")
-        self._awaiting_ack = True
-#         self._lbl_send_port.config(image=self._img_pending)
+            if self._preset_command == 'CFG-CFG - RESTORE FACTORY DEFAULTS':
+                confirmed = self._do_factory_reset()
+            elif self._preset_command == 'CFG-CFG - Save current configuration to RAM':
+                confirmed = self._do_save_config()
+            elif self._preset_command == 'CFG-INF - Turn ON all INF msgs':
+                self._do_set_inf(True)
+            elif self._preset_command == 'CFG-INF - Turn OFF all non-error INF msgs':
+                self._do_set_inf(False)
+            elif self._preset_command == 'CFG-MSG - Turn ON all LOG msgs':
+                self._do_set_log(True)
+            elif self._preset_command == 'CFG-MSG - Turn OFF all LOG msgs':
+                self._do_set_log(False)
+            elif self._preset_command == 'CFG-MSG - Turn ON all MON msgs':
+                self._do_set_mon(True)
+            elif self._preset_command == 'CFG-MSG - Turn OFF all MON msgs':
+                self._do_set_mon(False)
+            elif self._preset_command == 'CFG-MSG - Turn ON all RXM msgs':
+                self._do_set_rxm(True)
+            elif self._preset_command == 'CFG-MSG - Turn OFF all RXM msgs':
+                self._do_set_rxm(False)
+            elif self._preset_command == 'CFG-MSG - Turn ON minimum NMEA msgs':
+                self._do_set_minnmea()
+            elif self._preset_command == 'CFG-MSG - Turn ON all NMEA msgs':
+                self._do_set_allnmea(True)
+            elif self._preset_command == 'CFG-MSG - Turn OFF all NMEA msgs':
+                self._do_set_allnmea(False)
+            elif self._preset_command == 'CFG-MSG - Turn ON minimum UBX NAV msgs':
+                self._do_set_minNAV()
+            elif self._preset_command == 'CFG-MSG - Turn ON all UBX NAV msgs':
+                self._do_set_allNAV(True)
+            elif self._preset_command == 'CFG-MSG - Turn OFF all UBX NAV msgs':
+                self._do_set_allNAV(False)
+            elif self._preset_command == 'CFG-PRT - Poll Port config':
+                self._do_poll_prt()
+            elif self._preset_command == 'CFG-INF - Poll Info message config':
+                self._do_poll_inf()
+            else:
+                self._do_user_defined(self._preset_command)
+
+            if confirmed:
+                self.set_status("Command(s) sent", "blue")
+                self._awaiting_ack = True
+                self._lbl_send_preset.config(image=self._img_pending)
+            else:
+                self.set_status("Command(s) cancelled", "blue")
+
+        except Exception as err:  # pylint: disable=broad-except
+            self.set_status(f"Error {err}", "red")
+            self._lbl_send_preset.config(image=self._img_warn)
 
     def _do_poll_inf(self):
         '''
@@ -595,12 +528,12 @@ class UBXConfigDialog():
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
-            if msgtype[0:1] in (b'\xF0'):  # standard NMEA
+            if msgtype[0:1] == b'\xF0':  # standard NMEA
                 if msgtype in (b'\xF0\x00', b'\xF0\x02', b'\xF0\x03'):  # GGA, GSA, GSV
                     self._do_cfgmsg(msgtype, True)
                 else:
                     self._do_cfgmsg(msgtype, False)
-            if msgtype[0:1] in (b'\xF1'):  # proprietary NMEA
+            if msgtype[0:1] == b'\xF1':  # proprietary NMEA
                 self._do_cfgmsg(msgtype, False)
 
     def _do_set_minNAV(self):
@@ -609,7 +542,7 @@ class UBXConfigDialog():
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
-            if msgtype[0:1] in (b'\x01'):  # UBX-NAV
+            if msgtype[0:1] == b'\x01':  # UBX-NAV
                 if msgtype in (b'\x01\x07', b'\x01\x30'):  # NAV-PVT, NAV-SVINFO
                     self._do_cfgmsg(msgtype, True)
                 else:
@@ -646,18 +579,60 @@ class UBXConfigDialog():
         msg = UBXMessage('CFG', 'CFG-MSG', payload, SET)
         self.__app.serial_handler.serial_write(msg.serialize())
 
-    def _do_factory_reset(self):
+    def _do_factory_reset(self) -> bool:
         '''
-        Restore to factory defaults
+        Restore to factory defaults stored in battery-backed RAM
+        but display confirmation message box first.
         '''
 
-        clearMask = b'\x07\x00\x00\x00'
-        saveMask = b'\x00\x00\x00\x00'
-        loadMask = b'\x07\x00\x00\x00'
-        devicerMask = b'\x01'
-        payload = clearMask + saveMask + loadMask + devicerMask
-        msg = UBXMessage('CFG', 'CFG-CFG', payload, SET)
-        self.__app.serial_handler.serial_write(msg.serialize())
+        if messagebox.askokcancel(DLGRESET, DLGRESETCONFIRM):
+            clearMask = b'\x1f\x1f\x00\x00'
+            saveMask = b'\x00\x00\x00\x00'
+            loadMask = b'\x1f\x1f\x00\x00'
+            devicerMask = b'\x01'  # battery backed RAM
+            payload = clearMask + saveMask + loadMask + devicerMask
+            msg = UBXMessage('CFG', 'CFG-CFG', payload, SET)
+            self.__app.serial_handler.serial_write(msg.serialize())
+            return True
+
+        return False
+
+    def _do_save_config(self) -> bool:
+        '''
+        Save current configuration to battery-backed RAM
+        but display confirmation message box first.
+        '''
+
+        if messagebox.askokcancel(DLGSAVE, DLGSAVECONFIRM):
+            clearMask = b'\x00\x00\x00\x00'
+            saveMask = b'\x1f\x1f\x00\x00'
+            loadMask = b'\x00\x00\x00\x00'
+            devicerMask = b'\x01'  # battery backed RAM
+            payload = clearMask + saveMask + loadMask + devicerMask
+            msg = UBXMessage('CFG', 'CFG-CFG', payload, SET)
+            self.__app.serial_handler.serial_write(msg.serialize())
+            return True
+
+        return False
+
+    def _do_user_defined(self, command):
+        '''
+        Parse and send user-defined command(s).
+
+        This could result in any number of errors if the
+        uxbpresets file contains garbage, so there's a broad
+        catch-all-exceptions in the calling routine.
+        '''
+
+        seg = command.split(",")
+        for i in range(1, len(seg), 4):
+            ubx_class = seg[i].strip()
+            ubx_id = seg[i + 1].strip()
+            payload = seg[i + 2].strip()
+            mode = int(seg[i + 3].rstrip('\r\n'))
+            payload = bytes(bytearray.fromhex(payload))
+            msg = UBXMessage(ubx_class, ubx_id, payload, mode)
+            self.__app.serial_handler.serial_write(msg.serialize())
 
     def set_status(self, message, color="blue"):
         '''
@@ -666,7 +641,7 @@ class UBXConfigDialog():
 
         message = (message[:50] + '..') if len(message) > 50 else message
         self._lbl_status.config(fg=color)
-        self._status.set("  " + message)
+        self._status_preset.set("  " + message)
 
     def _on_exit(self, *args, **kwargs):
         '''
